/*
 * Cache Manager
 * 
 * V2.0.7
 */

/* global WebOSManager, SOAPClient, XPathResult, utils, base64js, LibraryManager, LayoutManager, ScheduleManager, ERRORS, XMLDocument, application, logic, Logger */

"use strict";

/**
 * @classdesc Provide cache functionality.
 * @type CacheManager
 * @version 2.0.7
 * @since 2.0.7
 */
var CacheManager = new function () {
  // <editor-fold defaultstate="collapsed" desc="Private variables">
  var v_LayoutDurations = {};
  var cycleSubList = [];
  var campaignOrder = {};
  var currentGroupKey = -1 ;
  var nextLayoutIdx;

  /**
   * Keep track of how many plays a layout did per hour.
   * @type {Object} Object with layout ids as keys and plays per hour and last (re)set time.
   */
  var maxPlaysPerHour = {};
  // </editor-fold>

  // <editor-fold defaultstate="collapsed" desc="Private functions">
  // </editor-fold>

  // <editor-fold defaultstate="collapsed" desc="Public functions">
  
  /**
   * Set the duration of the layout.
   * @param {String} id The id of the layout.
   * @param {String|Number} duration The duration of the layout in seconds.
   * @returns {undefined} 
   */
  this.setLayoutDuration = function(id , duration){
    v_LayoutDurations[id] = duration;
  };
  
  /**
   * Get the duration of the layout.
   * @param {String} id The id of the layout.
   * @param {String|Number} defaultDuration The duration of the layout in seconds if not previously set.
   * @returns {String|Number}  The duration of the layout in seconds.
   */
  this.getLayoutDuration = function(id , defaultDuration){
    if (Object.keys(v_LayoutDurations).indexOf(id)!==-1){
        return v_LayoutDurations[id];
    }
    return defaultDuration;
  };
  this.setCycleListSequence  = function (media) {
    var j;
    var pcycleref ;
    var mediaflag = 0;// 1 if media exits !

    // if (Object.keys(cycleSubList).indexOf(media.parentWidgetId)===-1){
    if (!cycleSubList[media.parentWidgetId]){
      cycleSubList[media.parentWidgetId] = {
        count: media.countplay,
        Israndom: media.IsRandom,
        counter: media.countplay,
        mediaIdx: -1,
        mediaId: []
      }
    }
    for (j = 0; j < cycleSubList[media.parentWidgetId].mediaId.length; j++) {
      if (cycleSubList[media.parentWidgetId].mediaId[j].mid === media.id) {
        mediaflag = 1;
        break;
      }
    }
    if (!mediaflag) {
      cycleSubList[media.parentWidgetId].mediaId.push({
        mid: media.id,
        media: media,
      })
    }
  }
  /**
   * Get the next widget from the cycle
   * @param {String} media The duration of the layout in seconds if not previously set.
   * @returns {String|Number}  The duration of the layout in seconds.
   */
  this.counter = function (media) {
    var parentWid = cycleSubList[media.parentWidgetId];
    if (parentWid.count == parentWid.counter) {
      parentWid.counter = 0;
    } else {
      parentWid.counter += 1;
    }
    return parentWid.counter;
  }
  this.getNextWidgetFromCycle = function (media) {
    var c;
    var current_mediaIdx = cycleSubList[media.parentWidgetId].mediaIdx;
    var allMedia = cycleSubList[media.parentWidgetId].mediaId;

    c = CacheManager.counter(media);

    if (c === 0 ){
      CacheManager.counter(media);
      if (cycleSubList[media.parentWidgetId].Israndom == "1") {
        current_mediaIdx = Math.floor(Math.random() * allMedia.length);
      } else {
        current_mediaIdx += 1;
        if (current_mediaIdx >= allMedia.length) {
          current_mediaIdx = 0;
        }
      }
    }
    cycleSubList[media.parentWidgetId].mediaIdx = current_mediaIdx;
    media = allMedia[current_mediaIdx].media;
    return media;
  };
  this.setCampaignOrder =function(layout){
    var j
    var layoutflag = 0;// 1 if layout exits !
    if (!campaignOrder[layout.groupKey]){
      campaignOrder[layout.groupKey] = {
        groupKey: layout.groupKey,
        count: layout.playCount,
        counter: layout.playCount,
        layoutIdx: -1,
        layouts: []
      }
    }else{
      campaignOrder[layout.groupKey].count = layout.playCount;
    }
    for (j = 0; j < campaignOrder[layout.groupKey].layouts.length; j++) {
      if (campaignOrder[layout.groupKey].layouts[j].layoutId === layout.file) {
        layoutflag = 1;
        break;
      }
    }

    if (!layoutflag) {
      campaignOrder[layout.groupKey].layouts.push({
        layoutId: layout.file,
        layout: layout
      })
      campaignOrder[layout.groupKey].counter = layout.playCount
    }
    //console.log("my campaign list is ready")
  };
  this.checkCampaignOrder = function (layouts){
    var cpkeys // campaigns ids
    var i,layout,campFound;
    var index,layoutfound;
    var l_index = [];
    for(i=0; i<layouts.length; i++){
      if(layouts[i].cyclePlayback == "1"){
        layout= layouts[i];
        CacheManager.setCampaignOrder(layout); // add in the list only the layouts that matter
      }
    }
    cpkeys = Object.keys(campaignOrder);
    for(i=0;i<cpkeys.length; i++){ //for every campaign
      campFound = 0;
      index = i;
      for (var j = 0; j < layouts.length; j++) { //for every layout
        if(layouts[j].cyclePlayback == "1") { //if they belong in a campaign
          layoutfound = 0
          if(cpkeys[i] == layouts[j].groupKey){
            campFound = 1;
            for (var k = 0; k < campaignOrder[cpkeys[i]].layouts.length; k++) { // By the way we should check if the layouts in the campaign matches with the layout we got from cms
              layoutfound = layouts.map(function(object){return object.file} ).indexOf(campaignOrder[cpkeys[i]].layouts[k].layoutId);
              if(layoutfound == -1){
                l_index.push(k);
              }
            }
            break;
          }
        }
      }
      if(!campFound){//remove the campaigns that didnt match with the layouts we should play
        delete campaignOrder[cpkeys[i]];
      }else{ // remove the layouts from the campaign, that we dodnt need any more
        if(l_index.length> 0){
          for (var j = (l_index.length -1); j >= 0; j--) {
            campaignOrder[cpkeys[i]].layouts.splice(l_index[j], 1);
            // campaignOrder[cpkeys[i]].layoutIdx = -1
          }
          campaignOrder[cpkeys[i]].counter = layout.playCount;
        }
      }
    }
  };
  this.checkLayout =  function(layout){
    var cpkeys,flag,i,id;
    id = layout.id
    flag = -1;
    cpkeys = Object.keys(campaignOrder);
    if(cpkeys.length >1){
      for(i=0;i<cpkeys.length; i++){
        var layouts = campaignOrder[cpkeys[i]].layouts
        for (var j = 0; j < layouts.length; j++) {
          if(layout.id ==layouts[j].layoutId){
            if(currentGroupKey == cpkeys[i] ){
              flag =0;
              break ;
            }else{
              flag =1 ;
              currentGroupKey = cpkeys[i]
            }
          }
        }
        if(flag === 0){
          break;
        }
      }
    }else if(cpkeys.length == 1){
      currentGroupKey = cpkeys[0];
      var layouts = campaignOrder[cpkeys[0]].layouts;
      if(layouts.length>1){
        if(layout.id ==layouts[0].layoutId) {
          flag=1;
        }else{
          for (var j = 1; j < layouts.length; j++) {
            if(layout.id ==layouts[j].layoutId){
              flag =0;
              break ;
            }else{ // else keep the current campaign id
              flag = -1 ;
            }
          }
          // currentGroupKey = 0;
        }
      }else{
        flag = -1;
      }
    }
    return flag;
  };
  this.removeBlackListedCycleLayout = function(layout){
    var index = campaignOrder[layout.groupkey].layouts.indexOf(layout.id);
    if (index > -1) { // only splice array when item is found
      campaignOrder[layout.groupkey].layouts.splice(index, 1); // 2nd parameter means remove one item only
    }
  };
  this.campaignCounter = function(layout){

    var campaignParent = campaignOrder[currentGroupKey];
    if (campaignParent.count ==campaignParent.counter) {
      campaignParent.counter = 0;
    } else {
      campaignParent.counter += 1;
    }
    return campaignParent.counter;
  };
  this.getNextLayoutFromCamp = function (layout) {
    var c,layoutID;
    var current_layoutIdx = campaignOrder[currentGroupKey].layoutIdx;
    var allLayouts = campaignOrder[currentGroupKey].layouts;


    c = CacheManager.campaignCounter(layout);

    if (c === 0 ){
      CacheManager.campaignCounter(layout);
      current_layoutIdx += 1;
      if (current_layoutIdx >= allLayouts.length) {
        current_layoutIdx = 0;
      }
    }
    campaignOrder[currentGroupKey].layoutIdx = current_layoutIdx;
    layoutID = allLayouts[current_layoutIdx].layoutId;
    return layoutID;
  };
  /**
   * Add a play count for the specific layout.
   * @param layoutId The id of the layout.
   */
  this.mpph_addPlay = function(layoutId){
    if (!maxPlaysPerHour[layoutId]){
      CacheManager.mpph_getPlays(layoutId);
    }
    maxPlaysPerHour[layoutId].plays += 1;

  };
  /**
   * Get the number of plays in the current hour.
   * @param layoutId The id of the layout.
   * @returns {Object} An object with the plays and the time started/reset.
   */
  this.mpph_getPlays = function(layoutId){
    if (!maxPlaysPerHour[layoutId]){
      maxPlaysPerHour[layoutId] = {
        plays : 0,
        time : new Date()
      }
    }

    return maxPlaysPerHour[layoutId];
  };

  /**
   * Reset the plays per hour and the time.
   * @param layoutId The id of the layout.
   */
  this.mpph_reset = function(layoutId){
    if (!maxPlaysPerHour[layoutId]){
      CacheManager.mpph_getPlays(layoutId);
    }else {
      maxPlaysPerHour[layoutId].plays = 0;
      maxPlaysPerHour[layoutId].time = new Date();
    }
  };

  // </editor-fold>
  // <editor-fold defaultstate="collapsed" desc="Native CMS Calls">
  
  // </editor-fold>
};